import { Server } from 'socket.io'
import { ExtendedCharacter, TSocket } from '../../utilities/types'
import ZoneRepository from '../../repositories/zoneRepository'
import { Character, Zone } from '@prisma/client'
import CharacterManager from '../../managers/characterManager'
import { gameLogger } from '../../utilities/logger'
import CharacterRepository from '../../repositories/characterRepository'

interface IResponse {
  zone: Zone
  characters: Character[]
}

export default class CharacterJoinEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('zone:character:join', this.handleCharacterJoin.bind(this))
  }

  private async handleCharacterJoin(callback: (response: IResponse) => void): Promise<void> {
    try {
      if (!this.socket.characterId) {
        gameLogger.error('zone:character:join error', 'Zone requested but no character id set')
        return
      }

      const character = await CharacterRepository.getById(this.socket.characterId as number)
      if (!character) {
        gameLogger.error('zone:character:join error', 'Character not found')
        return
      }

      const zone = await ZoneRepository.getById(character.zoneId)
      if (!zone) {
        gameLogger.error('zone:character:join error', 'Zone not found')
        return
      }

      CharacterManager.initCharacter(character as ExtendedCharacter)

      this.socket.join(zone.id.toString())

      // let other clients know of new character
      this.io.to(zone.id.toString()).emit('zone:character:join', character)

      // Log
      gameLogger.info(`User ${character.id} joined zone ${zone.id}`)

      // send over zone and characters to socket
      callback({ zone, characters: CharacterManager.getCharactersInZone(zone) })
    } catch (error: any) {
      gameLogger.error('zone:character:join error', error.message)
      this.socket.disconnect()
    }
  }
}
